""" Module to construct / parse bencoded data """


def parse_blist(bdata):
    """ Convert bencoded data to python list """
    
    blist = []

    if bdata[0:1] == b'l':
        bdata = bdata[1:]
    
    while bdata[0:1] != b'' and bdata[0:1] != b'e':
        
        parse_func = btype_dict.get(bdata[0:1], parse_bstring)
        elem, bdata = parse_func(bdata) 
    
        blist.append(elem)

    return blist, bdata[1:]


def parse_bdict(bdata):
    """ Convert bencoded data to python dict """

    bdict = {}

    if bdata[0:1] == b'd':
        bdata = bdata[1:]

    while bdata[0:1] != b'' and bdata[0:1] != b'e':
        
        parse_func = btype_dict.get(bdata[0:1], parse_bstring)
        key, bdata = parse_func(bdata)
        
        if bdata[0:1] == '' or bdata[0:1] == 'e':
            value = None
        else:
            parse_func = btype_dict.get(bdata[0:1], parse_bstring)
            value, bdata = parse_func(bdata)

        if key in bdict:
            raise KeyError("Multiple keys in bencoded dictionary")
        
        bdict[key] = value

    return bdict, bdata[1:]


def parse_bint(bdata):
    """ Convert bencoded data to int """
    
    end_pos = bdata.index(ord('e'))
    num_str = bdata[1:end_pos]
    bdata = bdata[end_pos + 1:]

    return int(num_str), bdata


def parse_bstring(bdata):
    """ Convert bencoded data to string """
    
    delim_pos = bdata.index(ord(':'))
    length = bdata[0:delim_pos]
    length = int(length) 
    
    delim_pos += 1
    bstring = bdata[delim_pos:delim_pos + length]
    bdata = bdata[delim_pos + length:]

    if len(bstring) != length:
        raise ValueError("Incorrect bencoded string length")

    return bstring, bdata


def decode(bdata):
    """ Parse data and return a list of objects """

    return parse_blist(bdata)[0]
    

btype_dict = {
    b'd' : parse_bdict,
    b'l' : parse_blist,
    b'i' : parse_bint
}


#setuptools.setup is looking at one argv parameter; to "build" and "install":
#python3 setup.py install

pkname='torra'

import pathlib
HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()

from setuptools import setup
setup(name=pkname,
	version='1.0.17',
	packages=[pkname],
	#opt
	python_requires='>=3',
	install_requires=["appdirs>=1.4.3",
		'python-libtorrent-bin>=1.2.9;platform_system=="Linux" and platform_machine=="x86_64"'],
	description='Torrent client',
	long_description=README,
	long_description_content_type="text/markdown",
	url='https://github.com/colin-i/tora',
	author='bot',
	author_email='costin.botescu@gmail.com',
	license='MIT',
	entry_points = {
		'console_scripts': [pkname+'='+pkname+'.main:main']
	}
)



from . import gtk
from . import meninfo
from . import torrent
from . import treesel
from . import layout
from . import next
k=gtk.k

timer=0

@gtk.CALLBACK0i
def fire():
	global timer
	k.gtk_widget_hide(menu)
	timer=0
	return False

@gtk.CALLBACK2
def click(tree,path):
	global timer
	if timer>0:
		k.g_source_remove(timer)
	k.gtk_widget_show(menu)
	torrent.sel(tree,path)
	timer=k.g_timeout_add(10000,fire,None)

def hide():
	global timer
	k.g_source_remove(timer)
	timer=0
	k.gtk_widget_hide(menu)

def remtor(i,iter):
	torrent.remsel(i)
	k.gtk_list_store_remove(layout.list,iter)
@gtk.CALLBACK
def remclick(tree):
	treesel.position__iter(remtor,tree)
	if k.gtk_tree_model_iter_n_children(layout.list,None)==0:
		next.unini(tree)
	hide()

@gtk.CALLBACK2
def infoclick(b,t):
	meninfo.act(b,t)
	hide()

def ini(parent,tree):
	global menu
	menu=k.gtk_box_new(gtk.GtkOrientation.GTK_ORIENTATION_HORIZONTAL,0)
	uni=chr(0x24D8).encode()
	b=k.gtk_button_new_with_label(uni)
	k.g_signal_connect_data(b,b"clicked",infoclick,tree,None,0)
	k.gtk_box_append(menu,b)
	uni=chr(0x1F5D1).encode()
	b=k.gtk_button_new_with_label(uni)
	k.g_signal_connect_data(b,b"clicked",remclick,tree,None,gtk.GConnectFlags.G_CONNECT_SWAPPED)
	k.gtk_box_append(menu,b)
	k.gtk_box_append(parent,menu)
	k.gtk_widget_hide(menu)
	k.g_signal_connect_data(tree,b"row-activated",click,None,None,0)
	k.gtk_tree_view_set_activate_on_single_click(tree,True)
